
   import java.awt.*;
   import java.awt.event.*;

    public class CannonWorld extends Frame
   {
      public static final int FrameWidth = 600;
      public static final int FrameHeight = 400;
   
      private int        angle;
      private String     message;
      private ChangingCannonBall cannonBall;
      private Scrollbar  slider;
   
       public CannonWorld()
      {
         setSize ( FrameWidth, FrameHeight );
         setTitle( "Cannon Game" );
      
         angle      = 45;
         message    = "Angle: " + angle;
         cannonBall = null;
      
         slider = new Scrollbar( Scrollbar.VERTICAL, angle, 5, 0, 90 );
         slider.addAdjustmentListener( new ScrollBarListener() );
         add( "East", slider );
      
         Button fire = new Button( "fire" );
         fire.addActionListener( new FireButtonListener() );
         add( "North", fire );
      
         Button changeColor = new Button( "change ball color" );
         changeColor.addActionListener( new ChangeColorButtonListener() );
         add( "West", changeColor );
      }
   
       public static int dy( int y )
      {
         return FrameHeight - y;
      }
   
       public void paint( Graphics g )
      {
         drawCannon    ( g );
         drawTarget    ( g );
         drawCannonBall( g );
         writeMessage  ( g );
      }
   
   /* -------------- helper methods -------------- */
   
       protected void drawCannon( Graphics g )
      {
	  int x = 120; // needed to be changed was x = 20
         int y = 15;
      
         double radianAngle = angle * Math.PI / 180.0;
      
         int lv = (int) (30 * Math.sin(radianAngle));
         int lh = (int) (30 * Math.cos(radianAngle));
         int sv = (int) (10 * Math.sin(radianAngle + Math.PI/2));
         int sh = (int) (10 * Math.cos(radianAngle + Math.PI/2));
      
         g.setColor(Color.green);
         g.drawLine(x, dy(y), x+lh, dy(y+lv));
         g.drawLine(x+lh, dy(y+lv), x+lh+sh, dy(y+lv+sv));
         g.drawLine(x+lh+sh, dy(y+lv+sv), x+sh, dy(y+sv));
         g.drawLine(x+sh, dy(y+sv), x, dy(y));
         g.drawOval(x-8, dy(y+10), 12, 12);
      }
   
       protected void drawTarget( Graphics g )
      {
         g.setColor(Color.red);
         g.fillRoundRect( FrameWidth-100, dy(12), 50, 10, 6, 6 );
      }
   
       protected void drawCannonBall( Graphics g )
      {
         if ( cannonBall != null )
         { 
            cannonBall.move();
            cannonBall.paint( g );
            
	    try {  Thread.sleep( 40 ); }
	    catch (InterruptedException e) { System.exit( 0 ); 	}
	    
       
            if ( dy(cannonBall.y()) > 0 ) 
               repaint();
            else
            {
               int targetX = FrameWidth - 100;
               if ( (cannonBall.x() > targetX) &&
               (cannonBall.x() < (targetX + 50)) )
                  message = "You Hit It!";
               else
                  message = "Missed!";
               cannonBall = null;
            }
         }
      }
   
       protected void writeMessage( Graphics g )
      {
         g.drawString( message, FrameWidth/2, FrameHeight/2 );
      }
   
   /* -------------- inner classes -------------- */
   
       private class FireButtonListener implements ActionListener
       {
          public void actionPerformed( ActionEvent e )
         {
            double radianAngle = angle * Math.PI / 180.0;
            double sinAngle    = Math.sin( radianAngle );
            double cosAngle    = Math.cos( radianAngle );
         
            cannonBall = new ChangingCannonBall (
  	    // was 20 + (int) (30 * cosAngle)
               120 + (int) (30 * cosAngle),
               dy(5+(int) (30 * sinAngle)),
               5, 12 * cosAngle, -12 * sinAngle );
            repaint();
         }
      }
       
       private class ChangeColorButtonListener implements ActionListener
       {
          public void actionPerformed( ActionEvent e )
         {
	     if (cannonBall != null) {
		 cannonBall.nextColor();
		 repaint();
	     } // end if
         }
      }
   
       private class ScrollBarListener implements AdjustmentListener
      {
          public void adjustmentValueChanged (AdjustmentEvent e)
         {
            angle   = slider.getValue();
            message = "Angle: " + angle;
            repaint();
         }
      }
   }
